package org.example.mybatisplus.tools;


import com.alibaba.fastjson2.JSON;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.rendering.PDFRenderer;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.util.Units;
import org.apache.poi.xwpf.usermodel.Document;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.apache.poi.xwpf.usermodel.XWPFParagraph;
import org.apache.poi.xwpf.usermodel.XWPFRun;
import org.example.mybatisplus.module.vo.Attachment;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTPageMar;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTPageSz;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTSectPr;

import javax.imageio.ImageIO;
import java.awt.Color;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Base64;
import java.util.List;

public class ImageUtils {
    public static List<String> convertPdfToImages(String pdfUrl) throws Exception {
        URL url = new URL(pdfUrl);
        InputStream is = url.openStream();
        PDDocument document = PDDocument.load(is);
        PDFRenderer renderer = new PDFRenderer(document);
        List<String> images = new ArrayList<>();
        for (int i = 0; i < document.getNumberOfPages(); i++) {
            BufferedImage image = renderer.renderImageWithDPI(i, 150);
            images.add(bufferedImageToBase64(image));
        }
        document.close();
        return images;
    }

    /**
     * BufferedImage转base64
     *
     * @param bufferedImage
     * @return
     */
    public static String bufferedImageToBase64(BufferedImage bufferedImage) {
        ByteArrayOutputStream stream = new ByteArrayOutputStream();
        try {
            // 设置图片格式
            ImageIO.write(bufferedImage, "jpg", stream);
        } catch (IOException e) {
            e.printStackTrace();
        }
        byte[] bytes = Base64.getEncoder().encode(stream.toByteArray());
        String base64 = new String(bytes);
        return base64;
    }
    public static BufferedImage convertSheetToImage(Sheet sheet) {
        // 计算表格尺寸
        int rows = sheet.getPhysicalNumberOfRows();
        int cols = sheet.getRow(0).getPhysicalNumberOfCells();
        int cellWidth = 100;
        int cellHeight = 20;
        int imageWidth = cols * cellWidth;
        int imageHeight = rows * cellHeight;

        BufferedImage image = new BufferedImage(imageWidth, imageHeight, BufferedImage.TYPE_INT_RGB);
        Graphics2D g = image.createGraphics();
        g.setColor(Color.WHITE);
        g.fillRect(0, 0, imageWidth, imageHeight);
        g.setColor(Color.BLACK);

        // 绘制表格内容
        for (int i = 0; i < rows; i++) {
            Row row = sheet.getRow(i);
            for (int j = 0; j < cols; j++) {
                Cell cell = row.getCell(j);
                String value = cell != null ? cell.toString() : "";
                g.drawString(value, j * cellWidth + 2, i * cellHeight + 15);
            }
        }
        g.dispose();
        return image;
    }
    public static String imgUrlToBase64(String imageUrl) {
        try {
            URL url = new URL(imageUrl);
            InputStream is = url.openStream();
            byte[] bytes = is.readAllBytes(); // 从流中读取所有字节
            is.close();
            return Base64.getEncoder().encodeToString(bytes);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }
    public static String getFileExtension(String path) {
        if (path != null && path.contains(".")) {
            String ext= path.substring(path.lastIndexOf(".") + 1);
            if(ext.indexOf("?")>0){
                ext = ext.substring(0,ext.indexOf("?"));
            }
            return ext.toLowerCase();
        }
        return "";
    }
    public static List<String> urlTobase64(String url) {
        // 常见的图片文件扩展名列表
        try {
            List<String> imageExtensions = Arrays.asList("jpg", "jpeg", "png", "gif", "bmp", "tiff", "svg");
            String ext= getFileExtension(url);
            if(imageExtensions.contains(ext)){
                List<String> list = new ArrayList<>();
                list.add(imgUrlToBase64(url));
                return list;
            }else if(ext.equals("pdf")){
                return convertPdfToImages(url);
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

        return null;
    }
    public static void main(String[] args) throws Exception {
        String pdfUrl = "http://wap.xiuqiuengineering.com/jijinhui1.pdf"; // 替换为你的图片URL
        String imageUrl = "https://yyc3-1321096020.cos.ap-beijing.myqcloud.com/declaration/20250313/dc8220a3a722486d9ccd632972a3ab15.png?X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Date=20250317T075010Z&X-Amz-SignedHeaders=host&X-Amz-Expires=86399&X-Amz-Credential=AKIDRilauFg4fDfE9B6tJoNxTvHSdovqXBfw%2F20250317%2Fap-beijing%2Fs3%2Faws4_request&X-Amz-Signature=e2b1cbe340254b2b63c97ffadeee183b52409f5fa77384e68ed7298b501f8666"; // 替换为你的图片URL
        Attachment attach = new Attachment();
        attach.setFileName("关联实体情况.png");
        attach.setFilePath(imageUrl);
        //List<String> base64Image = convertPdfToImages(imageUrl);
        System.out.println(attach.getBase64().get(0));
    }
    public static InputStream bufferedImageToInputStream(BufferedImage bufferedImage, String format) throws IOException {
        // 创建一个ByteArrayOutputStream来存储图像数据
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        // 使用ImageIO将BufferedImage写入到ByteArrayOutputStream中
        ImageIO.write(bufferedImage, format, baos);
        // 将ByteArrayOutputStream转换为InputStream，以便在其他地方使用
        return new ByteArrayInputStream(baos.toByteArray());
    }
    private static void insertImage1(XWPFDocument doc, BufferedImage image, String title) throws Exception {
        XWPFParagraph para = doc.createParagraph();
        XWPFRun run = para.createRun();
        // 将BufferedImage转为字节数组
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        ImageIO.write(image, "PNG", baos);
        byte[] imageData = baos.toByteArray();

        // 计算图片尺寸
        int widthEMU = Units.toEMU(image.getWidth());
        int heightEMU = Units.toEMU(image.getHeight());

        run.addPicture(new ByteArrayInputStream(imageData),
                XWPFDocument.PICTURE_TYPE_PNG,
                "image.png",
                widthEMU,
                heightEMU);
    }
    static void insertImage(XWPFDocument doc, XWPFParagraph p, String imagePath, XWPFRun placeholderRun) throws IOException {
        try (FileInputStream is = new FileInputStream(imagePath)) {
            XWPFRun run = p.createRun();


            // 图片文件路径
            File imageFile = new File(imagePath);
            // 读取图片文件到字节数组
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            byte[] buffer = new byte[4096];
            int bytesRead;
            while ((bytesRead = is.read(buffer)) != -1) {
                baos.write(buffer, 0, bytesRead);
            }
            byte[] imageBytes = baos.toByteArray();

            // 获取图片原始尺寸
            BufferedImage image = ImageIO.read(imageFile);
            int originalWidth = image.getWidth();
            int originalHeight = image.getHeight();

            // 获取页面设置
            CTSectPr sectPr = doc.getDocument().getBody().getSectPr();
            if (sectPr == null) {
                sectPr = doc.getDocument().getBody().addNewSectPr();
            }

            // 获取页面宽度和边距（单位为Twips）
            CTPageSz pageSize = sectPr.getPgSz();
            CTPageMar pageMar = sectPr.getPgMar();

            long pageWidthTwips = Long.parseLong(pageSize.getW().toString());
            long leftMarginTwips = Long.parseLong(pageMar.getLeft().toString());
            long rightMarginTwips = Long.parseLong(pageMar.getRight().toString());

            // 计算可用宽度（Twips）并转换为EMU
            long availableWidthTwips = pageWidthTwips - leftMarginTwips - rightMarginTwips;
            int targetWidthEMU = (int) (availableWidthTwips * Units.EMU_PER_DXA);

            // 计算目标高度（保持宽高比）
            int targetHeightEMU = (int) (targetWidthEMU * ((double) originalHeight / originalWidth));

            // 插入图片
            run.addPicture(new ByteArrayInputStream(imageBytes),
                    Document.PICTURE_TYPE_PNG,
                    imageFile.getName(),
                    targetWidthEMU,
                    targetHeightEMU);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}